/* 
Kit Queda Livre - Programa 2.
Este programa foi desenvolvido em linguagem PROCESSING para receber valores de tempo via USB e gerar um Histograma (gráfico de barras para distribuição de frequências). 

O Histograma é atualizado em tempo de obtenção de dados, conforme os valores são obtidos pela Plataforma ARDUINO. i

Antes de executar, verifique se o programa 1 foi devidamente gravado na plataforma ARDUINO.

Gera 'Histograma de tempos de queda' em tempo de execução - FT UNICAMP */

import processing.serial.*;

/* Declaração de Variáveis ----- */
PFont Fonte1;
color on = color(0); // Cor de fundo
color off = color(255); // cor dos textos e linhas

int ii, jj, kk; // Contadores [for]

/* grafico() ------------------- */
int divCol = 13; //quantidade de colunas
int divLin = 14; //quantidade de linhas
int widCol, heiLin; // widCol -> Largura padrão de cada Coluna, heiLin -> Altura padrão de cada Linha
int ColY; // Valor no EIXO Y onde estão escritas as LEGENDAS de tempo

float media = 181.5; //média dos valores de tempo
float sigma = 0.1; //desvio padrão dos valores de tempo
float tempo = 0;

String sMedia= str(media); //recebe a Legenda tipo float a ser impressa (corta os zeros a direita após a vírgula)

/* leitura() ------------------- */
int inputSerial = 0; // Leitura na Porta serial
// 0 para null, e 1 para novo tempo recebido

String sTempo; //Recebe o tempo em Segundos, da entrada Serial (Arduíno)
int lf = 10;
Serial myPort;

/* histograma() ---------------- */
int[] posCol = new int[13]; // Colunas -> Armazena valor do Eixo X da coluna ii
int posLin; // Linhas -> Armazena valor inicial do Eixo Y [mesmo para todas as colunas]

float[] tmin = new float[13]; // Colunas -> valor de tempo mínimo para a classe [13 classes]
int[] freq = new int[13]; // Linhas -> quantidade de linhas / frequencia absoluta das classes [13 classes]

PrintWriter output; // Armazena os valores em um arquivo de texto

void setup()
{ 
  /* Comunicação com a Entrada Serial --------------------------------------- */
  println(Serial.list());
   
  myPort = new Serial(this, Serial.list()[0], 9600);
  myPort.clear();
   
  sTempo = myPort.readStringUntil(lf);
  sTempo = null;
    
  /* Frequencia Absoluta das Classes [inicializando] ------------------------ */
  for(ii=0; ii < divCol; ii++)
    freq[ii] = 0;
  
  /* Inicializando o TEMPO MINIMO de cada intervalo de classe --------------- */
  
  output = createWriter("tempos_de_queda.txt"); // Arquivo de texto -> Valores de tempo obtidos
  
  /**/output.println("Media: "+media);
  /**/output.flush();
  
  /**/output.println("Sigma: "+sigma);
  /**/output.flush();
  
  /**/output.println("Tempos Minimos dos intervalos:");
  /**/output.flush();
  
  ii=(divCol / 2); // 13 / 2 = 6 [tipo numerico inteiro], ou seja, de 0 - 12, a classe central é a #6
  tmin[ii] = ((media-(0.5*sigma)) + media)/2; // Classe central recebe o valor medio [não pode ser feito dentro do FOR]
  
  /**/sTempo = str(tmin[ii]);
  /**/output.println(sTempo);
  /**/output.flush();
  
  jj = ii +1; // Próxima classe
  kk = ii -1; // Classe anterior
  
  for(ii=0; ii<3; ii++, jj++, kk--){
    tmin[jj] = ((media+(ii*sigma))+(media+((ii + 0.5)*sigma)))/2; // Média aritmética entre a classe ii e a próxima classe
    
    /**/sTempo=str(tmin[jj]);
    /**/output.println(sTempo);
    /**/output.flush();
    
    jj++;
    
    tmin[jj] = ((media+((ii + 1.0)*sigma))+(media+((ii + 0.5)*sigma)))/2;
    
    /**/sTempo=str(tmin[jj]);
    /**/output.println(sTempo);
    /**/output.flush();
    
    tmin[kk] = ((media-((ii + 1.0)*sigma))+(media-((ii + 0.5)*sigma)))/2; // Média aritmética entre a classe ii e a classe anterior
    
    /**/sTempo=str(tmin[kk]);
    /**/output.println(sTempo);
    /**/output.flush();
    kk--;
    
    if(kk > 0)
    {
      tmin[kk] = ((media-((ii + 1.5)*sigma))+(media-((ii + 1.0)*sigma)))/2;
      
      /**/sTempo=str(tmin[kk]);
      /**/output.println(sTempo);
      /**/output.flush();
    }
    else{
      tmin[0] = 0.0; // Tempo minimo da menor classe é nulo

      /**/sTempo=str(tmin[0]);
      /**/output.println(sTempo);
      /**/output.flush();
    }
  }
  
  /**/output.println("Valores obtidos experimentalmente:");
  /**/output.flush();

  sTempo=null;
  
  size(975, 680); // Determina as Dimensões da tela
  // e a quantidade de valores maximos a serem contidos
  // 975/15 = 65 -> 13 colunas + 2 colunas laterais [margens]
  // 680/17 = 40 -> 14 linhas + 2 linhas superiores + 1 linha inferior
  
  //Fonte1 = loadFont("Georgia-48.vlw");
  Fonte1 = loadFont("TimesNewRomanPSMT-48.vlw");
  
}


void draw()
{
  background(off);
  textFont(Fonte1, 36);
  fill(on);
  
  widCol = (width /(divCol +2)); // Largura Padrão das Colunas
  
  heiLin = (height /(divLin +3)); // Altura padrão das Linhas
  
  /*  divCol+=2; // acrescenta um espaço a direita e um a esquerda do grafico
      divLin+=3; // acrescenta dois espaços acima e um abaixo do grafico     */
  
  text("Histograma de tempos de queda", widCol, (2*heiLin)-(heiLin/2) );
  
  grafico(); // Desenha o gráfico [sem as barras e valores de tempo]
  
  leitura(); // Faz a leitura do tempo obtido na Porta Serial, mostra esse valor, e incrementa a Frequência Absoluta do intervalo classificado
  
  histograma(); // Mostra as barras com suas respectivas alturas, de acordo com a Frequencia absoluta de cada classe
}


/* Desenha o gráfico - Finalizado */
void grafico()
{
  color linha = color(200); // cor das linhas de borda
  textFont(Fonte1, 20);
  stroke(linha);
  
  /* Desenha as Linhas Horizontais */
  for (ii=2*heiLin; ii < height; ii+= heiLin )
  {
    if (ii > (2 * heiLin)) {
      line(( widCol -15 ), ii, width -widCol, ii); // Todas as outras Linhas Horizontais -> -15 faz as linhas um pouco mais compridas
      // afastamento eixo X a esquerda [fixo para todas as retas horizontais], afastamento eixo Y a esquerda, 
      // afastamento eixo X a direita [fixo para todas as retas horizontais], afastamento eixo Y a direita
    }
    else
      line( widCol, ii, width -widCol, ii); // Primeira Linha
  }

  /* Legendas do Eixo Y */
  jj = divLin; // contador de linhas [legenda]
  
  // ii -> Eixo Y da impressão da legendas, (heiLin*2) -> Eixo Y da primeira Legenda
  for (ii = (heiLin*2); (ii < height) && (jj >= 0); ii+= heiLin, jj-- )
    text(jj, widCol/3, ii);
  // jj -> Legenda da Linha, widCol/3 -> afastamento em X para direita, ii -> afastamento em Y [Linha correspondente]
  
  fill(off);

  /* Desenha as Linhas Verticais [apenas a primeira e a última] */
  line(widCol, 2*heiLin, widCol, height - heiLin);
  line((width -widCol), 2*heiLin, (width-widCol), height - heiLin);
  // afastamento eixo X em cima, afastamento eixo Y em cima [fixo para todas as retas verticais], 
  // afastamento eixo X em baixo, afastamento eixo Y em baixo [fixo para todas as retas verticais]
  
  /* Legendas do Eixo X */
  textFont(Fonte1, 24);
  fill(on);
  stroke(on);
  
  ColY = (height - (heiLin/3) );
  // (height - (heiLin/3) ) -> Posição eixo Y das Legendas das Colunas [fixo para todas as colunas]
  
  sMedia = str(media);
  text(sMedia, ( (width/2) - ( widCol/2 ) ), ColY); // Mediana
  
  
  sMedia = str(media + (sigma * 1.0));
  text(sMedia, (width/2) + widCol + (widCol/2), ColY); //68,24 % dos valores

  sMedia = str(media - (sigma * 1.0));
  text(sMedia, (width/2) - (widCol*2) - (widCol/2), ColY);

  sMedia = str(media + (sigma * 2.0));
  text(sMedia, (width/2) + (widCol/2) + (widCol*3), ColY); // 95,44% dos valores

  sMedia = str(media - (sigma * 2.0));
  text(sMedia, (width/2) - (widCol*4) - (widCol/2), ColY);

  sMedia = str(media + (sigma * 3.0));  
  text("> " + sMedia, (width - (2*widCol) ), ColY); // 99,74% dos valores

  sMedia = str(media - (sigma * 3.0));
  text("< " + sMedia, widCol, ColY);

  fill(off);
}

/* Faz a leitura da porta serial */
void leitura()
{
  
  if ( myPort.available() > 0) { 
   fill(on);
   sTempo = myPort.readStringUntil(lf);
   inputSerial = 1;
   }
  
  /* Mostra o Tempo obtido na tela ----------- */
  if (sTempo != null ) {
    
    fill(on);
    textFont(Fonte1, 24);
    
    tempo = float(sTempo);
    sTempo = trim(sTempo);
    text("Tempo = "+sTempo+" ms", (9*widCol), (2*heiLin)-(heiLin/2));
    
    /* Incremento das FREQUENCIAS ABSOLUTAS -> determina o intervalo de classe do tempo obtido */
    ii=0;
    while((ii < (divCol - 1) ) && (inputSerial == 1)){
      if(tempo < tmin[ii+1]) // se o tempo obtido for menor que o tempo minimo da próxima classe, então se classifica na classe ii
      {                      // *os valores de tempo muito pequenos serão classificados na primeira classe
        if(freq[ii] < divLin) // Altura máxima das Linhas
          freq[ii]++;
        output.println(sTempo);
        output.flush(); // Salva o valor de sTempo no arquivo de texto
        
        inputSerial = 0;
      }
      ii++;
    }
    if(inputSerial==1){ // Caso não estaja classificado em nenhuma classe, então se classificará na última [tempo maior ou igual a media + 3*sigma]
      if(freq[divCol-1] < divLin)
        freq[divCol-1]++;
      
      output.println(sTempo);
      output.flush(); // Salva o valor de sTempo no arquivo de texto
      
      inputSerial = 0;
    }
  }
  
  else
    fill(off);
}


/* Faz o preenchimento das barras com os valores de tempo Obtidos ------------ */
void histograma()
{
  color data = color(192, 157, 242); // Cor das Barras de frequência
  
  for(ii=0; ii < divCol; ii++)
    posCol[ii] = ( (ii + 1) * widCol); // posCol -> Recebe os valores de posição do EIXO X (Colunas)
  
  posLin = (divLin+2) * heiLin; // Valor inicial do EIXO Y para as barras [mesmo para todas]
  
  fill(data); // Cor de Preenchimento das Barras
  
  for(ii=0; ii < divCol; ii++)
    rect(posCol[ii], posLin -(freq[ii] * heiLin), widCol, freq[ii] * heiLin);
  // X [Posição da coluna onde começa a barra], Y [Posição da linha onde 'termina' a barra  - valor zero],
  // Width [largura - padrão para todos os intervalos], Height [Altura da barra - frequência absoluta de cada classe]
}
